@@ -13,6 +13,14 @@ from .ssh_config import parse_ssh_config
 
 
 def advanced_ssh_config_parse_options():
+    if False:
+        print('A' * 100)
+        sys.stderr.write('B' * 100 + '\n')
+        sys.stdout.write('C' * 100 + '\n')
+        logging.warn('D' * 100 + '\n')
+        logging.info('E' * 100)
+        logging.error('F' * 100)
+
     parser = optparse.OptionParser(usage='%prog [-v] [-l 9] -H host [-p 22]',
                                    version='%prog {0}'.format(__version__))
 
@@ -44,6 +52,9 @@ def advanced_ssh_config_parse_options():
     if len(args):
         raise ValueError('This program only takes options, not args')
 
+    if options.update_sshconfig and not options.hostname:
+        return options
+
     validate_host(options.hostname)
     if not options.port is None:
         validate_port(options.port)
