@@ -28,7 +28,8 @@ class TestAdvancedSshConfig(unittest.TestCase):
         self.assertEqual(routing['gateways'], ['direct'])
         self.assertEqual(routing['verbose'], True)
         self.assertEqual(routing['proxy_type'], 'nc')
-        self.assertEqual(routing['proxy_commands'][0], ['nc', '-v', '-w', 180, '-G', 5, 'test', 23])
+        self.assertEqual(routing['proxy_commands'][0],
+                         ['nc', '-v', '-w', 180, '-G', 5, 'test', 23])
 
     def test_routing_hostname_in_config(self):
         contents = """
@@ -46,7 +47,8 @@ port = 25
         self.assertEqual(routing['port'], 25)
         self.assertEqual(routing['hostname'], '1.2.3.4')
         self.assertEqual(routing['proxy_type'], 'nc')
-        self.assertEqual(routing['proxy_commands'][0], ['nc', '-v', '-w', 180, '-G', 5, '1.2.3.4', 25])
+        self.assertEqual(routing['proxy_commands'][0],
+                         ['nc', '-v', '-w', 180, '-G', 5, '1.2.3.4', 25])
 
     def test_routing_config_override(self):
         contents = """
@@ -63,7 +65,8 @@ port = 25
         self.assertEqual(routing['port'], 23)
         self.assertEqual(routing['hostname'], 'test.com')
         self.assertEqual(routing['proxy_type'], 'nc')
-        self.assertEqual(routing['proxy_commands'][0], ['nc', '-v', '-w', 180, '-G', 5, 'test.com', 23])
+        self.assertEqual(routing['proxy_commands'][0],
+                         ['nc', '-v', '-w', 180, '-G', 5, 'test.com', 23])
 
     def test_routing_via_two_other_hosts(self):
         advssh = AdvancedSshConfig(hostname='aaa.com/bbb.com/ccc.com')
@@ -71,7 +74,8 @@ port = 25
         self.assertEqual(routing['hostname'], 'aaa.com')
         self.assertEqual(routing['proxy_type'], 'nc')
         self.assertEqual(routing['gateways'], ['direct'])
-        self.assertEqual(routing['proxy_commands'][0], ['nc', '-w', 180, '-G', 5, 'aaa.com', 22])
+        self.assertEqual(routing['proxy_commands'][0],
+                         ['nc', '-w', 180, '-G', 5, 'aaa.com', 22])
         self.assertEqual(routing['gateway_route'], ['bbb.com', 'ccc.com'])
 
     def test_routing_via_two_other_hosts_with_config_one(self):
@@ -81,12 +85,14 @@ hostname = 1.2.3.4
 port = 25
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='ddd.com/eee.com', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='ddd.com/eee.com',
+                                   configfiles=[DEFAULT_CONFIG])
         routing = advssh.get_routing()
         self.assertEqual(routing['hostname'], '1.2.3.4')
         self.assertEqual(routing['proxy_type'], 'nc')
         self.assertEqual(routing['gateways'], ['direct'])
-        self.assertEqual(routing['proxy_commands'][0], ['nc', '-w', 180, '-G', 5, '1.2.3.4', 25])
+        self.assertEqual(routing['proxy_commands'][0],
+                         ['nc', '-w', 180, '-G', 5, '1.2.3.4', 25])
         self.assertEqual(routing['gateway_route'], ['eee.com'])
 
 
@@ -99,7 +105,8 @@ port = 25
 port = 24
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.prepare_sshconfig()
         self.assertEqual(len(config.keys()), 2)
         self.assertEqual(config['test'].host, 'test')
@@ -113,10 +120,13 @@ port = 24
 localforward = 1 2.3.4.5 6 \n 7 8.9.10.11 12
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.prepare_sshconfig()
         self.assertEqual(config['test'].host, 'test')
-        self.assertEqual(config['test'].config, [('localforward', '1 2.3.4.5 6'), ('localforward', '7 8.9.10.11 12')])
+        self.assertEqual(config['test'].config,
+                         [('localforward', '1 2.3.4.5 6'),
+                          ('localforward', '7 8.9.10.11 12')])
 
     def test_inherits(self):
         contents = """
@@ -129,12 +139,13 @@ inherits = aaa
 port = 23
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
-        self.assertEqual(config['aaa'].clean_config['user'], 'toto')
+        self.assertEqual(config['aaa'].clean_config['user'], ['toto'])
         self.assertEqual('port' in config['aaa'].clean_config, False)
-        self.assertEqual(config['bbb'].clean_config['user'], 'toto')
-        self.assertEqual(config['bbb'].clean_config['port'], '23')
+        self.assertEqual(config['bbb'].clean_config['user'], ['toto'])
+        self.assertEqual(config['bbb'].clean_config['port'], ['23'])
 
     def test_build_ssh_config(self):
         contents = """
@@ -147,7 +158,8 @@ inherits = aaa
 port = 23
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.prepare_sshconfig()
         arr = advssh.build_sshconfig()
         string = '\n'.join(arr)
@@ -188,7 +200,8 @@ hostname = 1.2.3.4
 user = toto
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.prepare_sshconfig()
         arr = advssh.build_sshconfig()
         string = '\n'.join(arr)
@@ -228,7 +241,8 @@ inherits = ccc
 port = 23
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
         def call():
             return config['bbb'].clean_config
@@ -248,10 +262,11 @@ tcpkeepalive = 42
 inherits = bbb
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
-        self.assertEqual(config['ccc'].clean_config['user'], 'toto')
-        self.assertEqual(config['ccc'].clean_config['tcpkeepalive'], '42')
+        self.assertEqual(config['ccc'].clean_config['user'], ['toto'])
+        self.assertEqual(config['ccc'].clean_config['tcpkeepalive'], ['42'])
 
     def test_inherits_override(self):
         contents = """
@@ -263,10 +278,11 @@ inherits = aaa
 user = titi
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
-        self.assertEqual(config['aaa'].clean_config['user'], 'toto')
-        self.assertEqual(config['bbb'].clean_config['user'], 'titi')
+        self.assertEqual(config['aaa'].clean_config['user'], ['toto'])
+        self.assertEqual(config['bbb'].clean_config['user'], ['titi'])
 
     def test_inherits_loop(self):
         contents = """
@@ -280,7 +296,8 @@ inherits = aaa
 inherits = bbb
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
         def call(key):
             return config[key].clean_config
@@ -294,7 +311,8 @@ inherits = bbb
 inherits = aaa
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
         def call(key):
             return config[key].clean_config
@@ -315,9 +333,127 @@ inherits = bbb
 [bbb]
 """
         set_config(contents)
-        advssh = AdvancedSshConfig(hostname='test', configfiles=[DEFAULT_CONFIG])
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
         config = advssh.config.full
-        self.assertEquals(config['aaa'].clean_config, {'user': 'toto', 'proxycommand': 'nc'})
+        self.assertEquals(config['aaa'].clean_config, {'user': ['toto'], 'proxycommand': ['nc']})
+
+    def test_comment_simple(self):
+        contents = """
+[test]
+comment = Hello
+"""
+        config = set_config(contents)
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
+        routing = advssh.get_routing()
+        self.assertEquals(routing['comment'], ['Hello'])
+
+    def test_comment_multiline(self):
+        contents = """
+[test]
+comment = Hello
+          World
+                         !
+
+port = 22
+"""
+        config = set_config(contents)
+        advssh = AdvancedSshConfig(hostname='test',
+                                   configfiles=[DEFAULT_CONFIG])
+        routing = advssh.get_routing()
+        self.assertEquals(routing['comment'], ['Hello', 'World', '!'])
+
+    def test_build_ssh_config_with_multiline_localforward_onliner(self):
+        contents = """
+[localhost]
+user = toto
+localforward = 1 2.3.4.5 6 \n 7 8.9.10.11 12
+port = 22
+"""
+        set_config(contents)
+        advssh = AdvancedSshConfig(hostname='localhost',
+                                   configfiles=[DEFAULT_CONFIG])
+        config = advssh.prepare_sshconfig()
+        arr = advssh.build_sshconfig()
+        string = '\n'.join(arr)
+        self.assertEquals(len(arr), 7)
+        dest = """
+Host localhost
+  localforward 1 2.3.4.5 6
+  localforward 7 8.9.10.11 12
+  port 22
+  user toto
+  # hostname localhost
+
+"""
+        self.assertEquals(string.strip(), dest.strip())
+
+
+    def test_build_ssh_config_with_multiline_localforward(self):
+        contents = """
+[localhost]
+user = toto
+localforward = 1 2.3.4.5 6
+               7 8.9.10.11 12
+port = 22
+"""
+        set_config(contents)
+        advssh = AdvancedSshConfig(hostname='localhost',
+                                   configfiles=[DEFAULT_CONFIG])
+        config = advssh.prepare_sshconfig()
+        arr = advssh.build_sshconfig()
+        string = '\n'.join(arr)
+        self.assertEquals(len(arr), 7)
+        dest = """
+Host localhost
+  localforward 1 2.3.4.5 6
+  localforward 7 8.9.10.11 12
+  port 22
+  user toto
+  # hostname localhost
+
+"""
+        self.assertEquals(string.strip(), dest.strip())
+
+    def test_build_ssh_config_with_multiline_comment(self):
+        contents = """
+[localhost]
+port = 22
+comment = .
+          .            O
+          .     _______O_
+          .    /       O \\
+          .   / _ _ O _ _ \\
+          .    |    _    |
+          .    | o | | o |
+          .    |___|_|___|
+          .
+user = toto
+"""
+        set_config(contents)
+        advssh = AdvancedSshConfig(hostname='localhost',
+                                   configfiles=[DEFAULT_CONFIG])
+        config = advssh.prepare_sshconfig()
+        arr = advssh.build_sshconfig()
+        string = '\n'.join(arr)
+        dest = """
+Host localhost
+  port 22
+  user toto
+  # comment .
+  # comment .            O
+  # comment .     _______O_
+  # comment .    /       O \\
+  # comment .   / _ _ O _ _ \\
+  # comment .    |    _    |
+  # comment .    | o | | o |
+  # comment .    |___|_|___|
+  # comment .
+  # hostname localhost
+
+"""
+        self.assertEquals(string.strip(), dest.strip())
 
 
     # FIXME: test_handle_custom_proxycommand
@@ -327,3 +463,4 @@ inherits = bbb
     # FIXME: test_dryrun
     # FIXME: test_verbose
     # FIXME: test_alias
+    # FIXME: test_build_ssh_config_real_hostname
